----------------------------------------------------------------------------------
-- implements a linear feedback shift register to generate a test pattern for the 
-- logic analyzer
--
----------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;



entity lfsr is
	Generic (
			CLK_DIV	: integer := 5;
			INITIAL : STD_LOGIC_VECTOR (7 downto 0) := "00000001" );
    Port ( rst : in  STD_LOGIC;
           clk : in  STD_LOGIC;
           dout : out  STD_LOGIC_VECTOR (7 downto 0));
end lfsr;

architecture Behavioral of lfsr is

	signal lfsrxDN, lfsrxDP	: std_logic_vector(7 downto 0) := INITIAL;
	signal cntxDN, cntxDP	: integer range 0 to 100000;

begin

	dout <= lfsrxDP;

	-- feed back logic, polynomial: x^8 + x^6 + x^5 + x^4 + 1
	process (lfsrxDP, cntxDP)
	begin
		if cntxDP = 0 then
			lfsrxDN <= lfsrxDP(6 downto 0) & (lfsrxDP(7) xor lfsrxDP(5) xor lfsrxDP(4) xor lfsrxDP(3));
		else
			lfsrxDN <= lfsrxDP;
		end if;
	end process;

	process (cntxDP)
	begin
		if cntxDP = CLK_DIV-1 then
			cntxDN <= 0;
		else
			cntxDN <= cntxDP + 1;
		end if;
	end process;
	
	-- implement the registers
	process(clk, rst)
	begin
		if rst='1' then
			lfsrxDP	<= INITIAL;
			cntxDP	<= 0;
		else 
			if clk'event and clk='1' then
				lfsrxDP	<= lfsrxDN;
				cntxDP 	<= cntxDN;
			end if;
		end if;
	end process;
	
end Behavioral;

