							
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
 
-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--USE ieee.numeric_std.ALL;
 
ENTITY LATop_tb IS
END LATop_tb;
 
ARCHITECTURE behavior OF LATop_tb IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
    COMPONENT LATop
	Generic (
		IS_SIMULATION	: string := "FALSE";
		SYS_TIMER_TOP	: integer := (40000000 / 100)-1 );
    PORT(
         xtalClockP : IN  std_logic;
         xtalClockN : IN  std_logic;
         resetSwitch : IN  std_logic;
         exClockxCI : IN  std_logic;
         inputChxDI : IN  std_logic_vector(31 downto 0);
         testOutputxSO : OUT  std_logic;
         UartTxxDO : OUT  std_logic;
         UartRxxDI : IN  std_logic;
         UartRtsxSO : OUT  std_logic;
         UartCtsxSI : IN  std_logic;
         PHY_COL : IN  std_logic;
         PHY_CRS : IN  std_logic;
         PHY_INT : IN  std_logic;
         PHY_MDC : OUT  std_logic;
         PHY_MDIO : INOUT  std_logic;
         PHY_RESET : OUT  std_logic;
         PHY_RXCLK : IN  std_logic;
         PHY_RXCTL_RXDV : IN  std_logic;
         PHY_RXD : IN  std_logic_vector(7 downto 0);
         PHY_RXER : IN  std_logic;
         PHY_TXCLK : IN  std_logic;
         PHY_TXCTL_TXEN : OUT  std_logic;
         PHY_TXC_GTXCLK : IN  std_logic;
         PHY_TXD : OUT  std_logic_vector(7 downto 0);
         PHY_TXER : OUT  std_logic;
         MemDQxDIO : INOUT  std_logic_vector(15 downto 0);
         MemAdrxDO : OUT  std_logic_vector(12 downto 0);
         MemBaxDO : OUT  std_logic_vector(2 downto 0);
         MemRasxEO : OUT  std_logic;
         MemCasxEO : OUT  std_logic;
         MemWexEO : OUT  std_logic;
         MemOdtxO : OUT  std_logic;
         MemCkexEO : OUT  std_logic;
         MemClkxCO : OUT  std_logic;
         MemClkxCOB : OUT  std_logic;
         MemLDqsxEO : INOUT  std_logic;
         MemLDqsxEOB : INOUT  std_logic;
         MemUDqsxEO : INOUT  std_logic;
         MemUDqsxEOB : INOUT  std_logic;
         MemLdmxEO : OUT  std_logic;
         MemUdmxEO : OUT  std_logic;
         RzqxIO : INOUT  std_logic;
         ZioxIO : INOUT  std_logic;
         LedxDO : OUT  std_logic_vector(3 downto 0)
        );
    END COMPONENT;
    
 
	Component ddr2
	Port (
		ck		: in  std_logic;
		ck_n	: in  std_logic;
		cke		: in  std_logic;
		cs_n	: in  std_logic;
		ras_n	: in  std_logic;
		cas_n	: in  std_logic;
		we_n	: in  std_logic;
		dm_rdqs	: inout std_logic_vector(1 downto 0);  
		ba		: in  std_logic_vector(2 downto 0);
		addr	: in  std_logic_vector(12 downto 0);
		dq		: inout std_logic_vector(15 downto 0);
		dqs		: inout std_logic_vector(1 downto 0);
		dqs_n	: inout std_logic_vector(1 downto 0);
		rdqs_n	: out std_logic_vector(1 downto 0);
		odt		: in  std_logic
	);
	end component;
	
	-- testbench simulating a ethernet phy supplied along with the mac
	component eth_phy is -- // This PHY model simulate simplified Intel LXT971A PHY
	port (
        -- COMMON
        m_rst_n_i	: in  std_logic;
		-- MAC TX
        mtx_clk_o	: out std_logic;
        mtxd_i		: in  std_logic_vector(3 downto 0);
        mtxen_i		: in  std_logic;
        mtxerr_i	: in  std_logic;
		-- MAC RX
        mrx_clk_o	: out std_logic;
        mrxd_o		: out std_logic_vector(3 downto 0);
        mrxdv_o		: out std_logic;
        mrxerr_o	: out std_logic;
		-- common mii
        mcoll_o		: out std_logic;
        mcrs_o		: out std_logic;
        -- MIIM
        mdc_i		: in  std_logic;
        md_io		: inout std_logic;
        -- SYSTEM
		phy_log		: in  std_logic_vector(31 downto 0));
	end component;
	
	--Inputs
	signal xtalClockP : std_logic := '0';
	signal xtalClockN : std_logic := '1';
	signal resetSwitch : std_logic := '1';
	signal exClockxCI : std_logic := '0';
	signal inputChxDI : std_logic_vector(31 downto 0) := (others => '0');
	signal UartRxxDI : std_logic := '0';
	signal UartCtsxSI : std_logic := '0';
	signal PHY_COL : std_logic := '0';
	signal PHY_CRS : std_logic := '0';
	signal PHY_INT : std_logic := '0';
	signal PHY_RXCLK : std_logic := '0';
	signal PHY_RXCTL_RXDV : std_logic := '0';
	signal PHY_RXD : std_logic_vector(7 downto 0) := (others => '0');
	signal PHY_RXER : std_logic := '0';
	signal PHY_TXCLK : std_logic := '0';
	signal PHY_TXC_GTXCLK : std_logic := '0';

	--BiDirs
	signal PHY_MDIO : std_logic;
	signal MemDQxDIO : std_logic_vector(15 downto 0);
	signal MemLDqsxEO : std_logic;
	signal MemLDqsxEOB : std_logic;
	signal MemUDqsxEO : std_logic;
	signal MemUDqsxEOB : std_logic;
	signal RzqxIO : std_logic;
	signal ZioxIO : std_logic;

 	--Outputs
	signal testOutputxSO : std_logic;
	signal UartTxxDO : std_logic;
	signal UartRtsxSO : std_logic;
	signal PHY_MDC : std_logic;
	signal PHY_RESET : std_logic;
	signal PHY_TXCTL_TXEN : std_logic;
	signal PHY_TXD : std_logic_vector(7 downto 0);
	signal PHY_TXER : std_logic;
	signal MemAdrxDO : std_logic_vector(12 downto 0);
	signal MemBaxDO : std_logic_vector(2 downto 0);
	signal MemRasxEO : std_logic;
	signal MemCasxEO : std_logic;
	signal MemWexEO : std_logic;
	signal MemOdtxO : std_logic;
	signal MemCkexEO : std_logic;
	signal MemClkxCO : std_logic;
	signal MemClkxCOB : std_logic;
	signal MemLdmxEO : std_logic;
	signal MemUdmxEO : std_logic;
	signal LedxDO : std_logic_vector(3 downto 0);
	
	-- signals needed to connect the Micron DDR2 model, unused for Elpida model
	signal dm		: std_logic_vector(1 downto 0);
	signal dqs		: std_logic_vector(1 downto 0);
	signal dqs_n	: std_logic_vector(1 downto 0);

   -- Clock period definitions
   constant BOARD_CLOCK_PERIOD : time := 5 ns;
   
BEGIN
 
	-- Instantiate the Unit Under Test (UUT)
	uut: LATop 
	Generic Map (
		IS_SIMULATION	=> "TRUE",
		SYS_TIMER_TOP	=> 100 )
	PORT MAP (
		xtalClockP => xtalClockP,
		xtalClockN => xtalClockN,
		resetSwitch => resetSwitch,
		exClockxCI => exClockxCI,
		inputChxDI => inputChxDI,
		testOutputxSO => testOutputxSO,
		UartTxxDO => UartTxxDO,
		UartRxxDI => UartRxxDI,
		UartRtsxSO => UartRtsxSO,
		UartCtsxSI => UartCtsxSI,
		PHY_COL => PHY_COL,
		PHY_CRS => PHY_CRS,
		PHY_INT => PHY_INT,
		PHY_MDC => PHY_MDC,
		PHY_MDIO => PHY_MDIO,
		PHY_RESET => PHY_RESET,
		PHY_RXCLK => PHY_RXCLK,
		PHY_RXCTL_RXDV => PHY_RXCTL_RXDV,
		PHY_RXD => PHY_RXD,
		PHY_RXER => PHY_RXER,
		PHY_TXCLK => PHY_TXCLK,
		PHY_TXCTL_TXEN => PHY_TXCTL_TXEN,
		PHY_TXC_GTXCLK => PHY_TXC_GTXCLK,
		PHY_TXD => PHY_TXD,
		PHY_TXER => PHY_TXER,
		MemDQxDIO => MemDQxDIO,
		MemAdrxDO => MemAdrxDO,
		MemBaxDO => MemBaxDO,
		MemRasxEO => MemRasxEO,
		MemCasxEO => MemCasxEO,
		MemWexEO => MemWexEO,
		MemOdtxO => MemOdtxO,
		MemCkexEO => MemCkexEO,
		MemClkxCO => MemClkxCO,
		MemClkxCOB => MemClkxCOB,
		MemLDqsxEO => MemLDqsxEO,
		MemLDqsxEOB => MemLDqsxEOB,
		MemUDqsxEO => MemUDqsxEO,
		MemUDqsxEOB => MemUDqsxEOB,
		MemLdmxEO => MemLdmxEO,
		MemUdmxEO => MemUdmxEO,
		RzqxIO => RzqxIO,
		ZioxIO => ZioxIO,
		LedxDO => LedxDO
	);

	-- board clock process 
	xtalClockN	<= not xtalClockP;
	xtalClock : process
	begin
		xtalClockP	<= '1';
		wait for BOARD_CLOCK_PERIOD / 2;
		xtalClockP	<= '0';
		wait for BOARD_CLOCK_PERIOD / 2;
	end process;
 

	-- Stimulus process
	stim_proc: process
	begin		
		-- hold reset state
		wait for 10 ns;	
		resetSwitch <= '0';
		-- Note: we have no stimulus here, the uC firmware loaded into the
		-- program memory is the stimulus as we want to see how firmware
		-- and hardware interact.      
		wait;
	end process;

	-- ddr2 memory model
	-- Note: elpida provides an encrypted model only, so modelsim or another
	-- proprietary tool is need. Instead a model from micron can be used as
	-- a substitute.
	dm 		<= MemUdmxEO & MemLdmxEO;
	dqs 	<= MemUDqsxEO & MemLDqsxEO;
	dqs_n	<= MemUDqsxEOB & MemLDqsxEOB;
	
--	ddr2ram : ede1116
	ddr2ram : ddr2
	PORT MAP (
		-- ports of elpida ede1116 memory model
--		cas_n => MemCasxEO,
--		ck => MemClkxCO,
--		ck_n => MemClkxCOB,
--		cke => MemCkexEO,
--		cs_n => '0',
--		ldm => MemLdmxEO,
--		odt => MemOdtxO,
--		ras_n => MemRasxEO,
--		udm => MemUdmxEO,
--		we_n => MemWexEO,
--		a => MemAdrxDO,
--		ba => MemBaxDO,
--		ldqs => MemLDqsxEO,
--		ldqs_n => MemLDqsxEOB,
--		udqs => MemUDqsxEO,
--		udqs_n => MemUDqsxEOB,
--		dq => MemDQxDIO

		-- ports of Micron ddr2 memory model
		ck => MemClkxCO,
		ck_n => MemClkxCOB,
		cke => MemCkexEO,
		cs_n => '0',
		ras_n => MemRasxEO,	
		cas_n => MemCasxEO,
		we_n => MemWexEO,
		dm_rdqs => dm,
		ba => MemBaxDO,
		addr => MemAdrxDO,
		dq => MemDQxDIO,
		dqs => dqs,
		dqs_n => dqs_n,
		rdqs_n => open,
		odt => MemOdtxO
	);
	
	-- phy model
	eth_phy_inst : eth_phy 
	port map (
        -- COMMON
        m_rst_n_i	=> PHY_RESET,
		-- MAC TX
        mtx_clk_o	=> PHY_TXCLK,
        mtxd_i		=> PHY_TXD(3 downto 0),
        mtxen_i		=> PHY_TXCTL_TXEN,
        mtxerr_i	=> PHY_TXER,
		-- MAC RX
        mrx_clk_o	=> PHY_RXCLK,
        mrxd_o		=> PHY_RXD(3 downto 0),
        mrxdv_o		=> PHY_RXCTL_RXDV,
        mrxerr_o	=> PHY_RXER,
		-- common mii
        mcoll_o		=> PHY_COL,
        mcrs_o		=> PHY_CRS,
        -- MIIM
        mdc_i		=> PHY_MDC,
        md_io		=> PHY_MDIO,
        -- SYSTEM
		phy_log		=> x"00000000"
	);

END;
