			-- TestBench Template 

  LIBRARY ieee;
  USE ieee.std_logic_1164.ALL;
  USE ieee.numeric_std.ALL;

  ENTITY testbench IS
  END testbench;

  ARCHITECTURE behavior OF testbench IS 

	component SoC is
	Generic (
			SYS_TIMER_TOP	: integer := (40000000 / 100)-1
			);
	Port ( 	xtalClockP : in  STD_LOGIC;
			xtalClockN : in std_logic;
			resetSwitch : in  STD_LOGIC;
	--	   	Int0xSI : in std_logic;
			UartTxxDO	: out std_logic;
			UartRxxDI	: in  std_logic;
			UartRtsxSO	: out std_logic;
			UartCtsxSI	: in  std_logic;
			PHY_COL			: in  std_logic;
			PHY_CRS			: in  std_logic;
			PHY_INT			: in  std_logic;
			PHY_MDC			: out std_logic;
			PHY_MDIO		: inout std_logic;
			PHY_RESET		: out std_logic;
			PHY_RXCLK		: in  std_logic;
			PHY_RXCTL_RXDV 	: in  std_logic;
			PHY_RXD			: in  std_logic_vector(7 downto 0);
			PHY_RXER		: in  std_logic;
			PHY_TXCLK		: in  std_logic;
			PHY_TXCTL_TXEN 	: out std_logic;
			PHY_TXC_GTXCLK	: in  std_logic;
			PHY_TXD			: out std_logic_vector(7 downto 0);
			PHY_TXER		: out std_logic;
           LedxDO : out  STD_LOGIC_VECTOR (3 downto 0));
	end component;

	-- testbench simulating a ethernet phy supplied along with the mac
	component eth_phy is -- // This PHY model simulate simplified Intel LXT971A PHY
	port (
        -- COMMON
        m_rst_n_i	: in  std_logic;
		-- MAC TX
        mtx_clk_o	: out std_logic;
        mtxd_i		: in  std_logic_vector(3 downto 0);
        mtxen_i		: in  std_logic;
        mtxerr_i	: in  std_logic;
		-- MAC RX
        mrx_clk_o	: out std_logic;
        mrxd_o		: out std_logic_vector(3 downto 0);
        mrxdv_o		: out std_logic;
        mrxerr_o	: out std_logic;
		-- common mii
        mcoll_o		: out std_logic;
        mcrs_o		: out std_logic;
        -- MIIM
        mdc_i		: in  std_logic;
        md_io		: inout std_logic;
        -- SYSTEM
		phy_log		: in  std_logic_vector(31 downto 0));
	end component;

	SIGNAL ClkxC :  std_logic := '0';
	signal ClkxCB : std_logic := '1';
    SIGNAL RstxR :  std_logic := '1';
	signal Int0xSI : std_logic := '0';
	SIGNAL LedxDO : std_logic_vector(3 downto 0);
    signal UartTxxDO	: std_logic;
	signal UartRxxDI	: std_logic := '1';
	signal UartRtsxSO	: std_logic;
	signal UartCtsxSI	: std_logic := '1';
	signal PHY_COL			: std_logic;
	signal PHY_CRS			: std_logic;
	signal PHY_INT			: std_logic;
	signal PHY_MDC			: std_logic;
	signal PHY_MDIO			: std_logic;
	signal PHY_RESET		: std_logic;
	signal PHY_RXCLK		: std_logic;
	signal PHY_RXCTL_RXDV 	: std_logic;
	signal PHY_RXD			: std_logic_vector(7 downto 0) := (others => '0');
	signal PHY_RXER			: std_logic;
	signal PHY_TXCLK		: std_logic;
	signal PHY_TXCTL_TXEN 	: std_logic;
	signal PHY_TXC_GTXCLK	: std_logic;
	signal PHY_TXD			: std_logic_vector(7 downto 0);
	signal PHY_TXER			: std_logic;
	
	signal PhyLogxD			: std_logic_vector(31 downto 0) := (others => '0');

  BEGIN

   ClkxCB <= not ClkxC;
	
	-- loop back uart data
	UartRxxDI <= UartTxxDO;
	
	uut: SoC 
	GENERIC MAP (
		SYS_TIMER_TOP => 100 )
	PORT MAP(
		xtalClockP 	=> ClkxC,
		xtalClockN	=> ClkxCB,
		resetSwitch => RstxR,
		UartTxxDO	=> UartTxxDO,
		UartRxxDI	=> UartRxxDI,
		UartRtsxSO	=> UartRtsxSO,
		UartCtsxSI	=> UartCtsxSI,
		PHY_COL			=> PHY_COL,
		PHY_CRS			=> PHY_CRS,
		PHY_INT			=> PHY_INT,
		PHY_MDC			=> PHY_MDC,
		PHY_MDIO		=> PHY_MDIO,
		PHY_RESET		=> PHY_RESET,
		PHY_RXCLK		=> PHY_RXCLK,
		PHY_RXCTL_RXDV 	=> PHY_RXCTL_RXDV,
		PHY_RXD			=> PHY_RXD,
		PHY_RXER		=> PHY_RXER,
		PHY_TXCLK		=> PHY_TXCLK,
		PHY_TXCTL_TXEN 	=> PHY_TXCTL_TXEN,
		PHY_TXC_GTXCLK	=> PHY_TXC_GTXCLK,
		PHY_TXD			=> PHY_TXD,
		PHY_TXER		=> PHY_TXER,
	--	Int0xSI		=> Int0xSI,
		LedxDO  	=> LedxDO
    );

	
	-- phy model
	eth_phy_inst : eth_phy 
	port map (
        -- COMMON
        m_rst_n_i	=> PHY_RESET,
		-- MAC TX
        mtx_clk_o	=> PHY_TXCLK,
        mtxd_i		=> PHY_TXD(3 downto 0),
        mtxen_i		=> PHY_TXCTL_TXEN,
        mtxerr_i	=> PHY_TXER,
		-- MAC RX
        mrx_clk_o	=> PHY_RXCLK,
        mrxd_o		=> PHY_RXD(3 downto 0),
        mrxdv_o		=> PHY_RXCTL_RXDV,
        mrxerr_o	=> PHY_RXER,
		-- common mii
        mcoll_o		=> PHY_COL,
        mcrs_o		=> PHY_CRS,
        -- MIIM
        mdc_i		=> PHY_MDC,
        md_io		=> PHY_MDIO,
        -- SYSTEM
		phy_log		=> PhyLogxD
	);
	
	clk: process
	begin
		ClkxC <= '0';
		wait for 2500 ps;
		ClkxC <= '1';
		wait for 2500 ps;
	end process;


     tb : PROCESS
     BEGIN
		RstxR <= '1';
        wait for 100 ns; -- wait until global set/reset completes
		RstxR <= '0';
        -- Add user defined stimulus here
        wait; -- will wait forever
     END PROCESS tb;

     intGenTb : process
	 begin
		wait for 2 us;
		Int0xSI <= '1';
		wait for 50 ns;
		Int0xSI <= '0';
		wait;
	 end process;

  END;
