
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
 
-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--USE ieee.numeric_std.ALL;
 
ENTITY dataBusTb IS
END dataBusTb;
 
ARCHITECTURE behavior OF dataBusTb IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
    COMPONENT dataBus
    PORT(
         ClkxCI : IN  std_logic;
         ClkxCIB : IN  std_logic;
         RstxRI : IN  std_logic;
         CpuCycxSI : IN  std_logic;
         CpuStbxSI : IN  std_logic;
         CpuWexSI : IN  std_logic;
         CpuAdrxDI : IN  std_logic_vector(15 downto 0);
         CpuDatxDI : IN  std_logic_vector(7 downto 0);
         CpuDatxDO : OUT  std_logic_vector(7 downto 0);
         CpuAckxSO : OUT  std_logic;
         CpuErrxSO : OUT  std_logic;
         CpuRtyxSO : OUT  std_logic;
         PhyTxClkxCI : IN  std_logic;
         PhyTxdxDO : OUT  std_logic_vector(3 downto 0);
         PhyTxEnxSO : OUT  std_logic;
         PhyTxErrxSO : OUT  std_logic;
         PhyRxClkxCI : IN  std_logic;
         PhyRxdxDI : IN  std_logic_vector(3 downto 0);
         PhyRxDvxSI : IN  std_logic;
         PhyRxErrxSI : IN  std_logic;
         PhyColxSI : IN  std_logic;
         PhyCrsxSI : IN  std_logic;
         PhyMdcxSO : OUT  std_logic;
         PhyMdioxDI : IN  std_logic;
         PhyMdioxDO : OUT  std_logic;
         PhyMdioEnxSO : OUT  std_logic;
         EthIntxSO : OUT  std_logic
        );
    END COMPONENT;
    
	-- testbench simulating a ethernet phy supplied along with the mac
	component eth_phy is -- // This PHY model simulate simplified Intel LXT971A PHY
	port (
        -- COMMON
        m_rst_n_i	: in  std_logic;
		-- MAC TX
        mtx_clk_o	: out std_logic;
        mtxd_i		: in  std_logic_vector(3 downto 0);
        mtxen_i		: in  std_logic;
        mtxerr_i	: in  std_logic;
		-- MAC RX
        mrx_clk_o	: out std_logic;
        mrxd_o		: out std_logic_vector(3 downto 0);
        mrxdv_o		: out std_logic;
        mrxerr_o	: out std_logic;
		-- common mii
        mcoll_o		: out std_logic;
        mcrs_o		: out std_logic;
        -- MIIM
        mdc_i		: in  std_logic;
        md_io		: inout std_logic;
        -- SYSTEM
		phy_log		: in  std_logic_vector(31 downto 0));
	end component;

   --Inputs
	signal ClkxCI : std_logic := '0';
	signal ClkxCIB : std_logic := '0';
	signal RstxRI : std_logic := '1';
	signal CpuCycxSI : std_logic := '0';
	signal CpuStbxSI : std_logic := '0';
	signal CpuWexSI : std_logic := '0';
	signal CpuAdrxDI : std_logic_vector(15 downto 0) := (others => '0');
	signal CpuDatxDI : std_logic_vector(7 downto 0) := (others => '0');
	signal PhyTxClkxCI : std_logic := '0';
	signal PhyRxClkxCI : std_logic := '0';
	signal PhyRxdxDI : std_logic_vector(3 downto 0) := (others => '0');
	signal PhyRxDvxSI : std_logic := '0';
	signal PhyRxErrxSI : std_logic := '0';
	signal PhyColxSI : std_logic := '0';
	signal PhyCrsxSI : std_logic := '0';
	signal PhyMdioxDI : std_logic := '0';

 	--Outputs
	signal CpuDatxDO : std_logic_vector(7 downto 0);
	signal CpuAckxSO : std_logic;
	signal CpuErrxSO : std_logic;
	signal CpuRtyxSO : std_logic;
	signal PhyTxdxDO : std_logic_vector(3 downto 0);
	signal PhyTxEnxSO : std_logic;
	signal PhyTxErrxSO : std_logic;
	signal PhyMdcxSO : std_logic;
	signal PhyMdioxDO : std_logic;
	signal PhyMdioEnxSO : std_logic;
	signal EthIntxSO : std_logic;

	signal PhyMdioxD : std_logic;
	signal PhyLogxD	 : std_logic_vector(31 downto 0) := x"00000000";

	-- Clock period definitions
	constant ClkxCI_period : time := 25 ns;
	
	signal WbData 	: std_logic_vector(31 downto 0) := (others => '0');
	signal WbAdr  	: std_logic_vector(15 downto 0) := (others => '0');
	signal WbStart	: std_logic := '0';
	signal WbDone	: std_logic := '0';
	
	signal OwnMacAdr : std_logic_vector(47 downto 0) := x"12345678ABCD";
	
BEGIN
 
	-- Instantiate the Unit Under Test (UUT)
   uut: dataBus PORT MAP (
          ClkxCI => ClkxCI,
          ClkxCIB => ClkxCIB,
          RstxRI => RstxRI,
          CpuCycxSI => CpuCycxSI,
          CpuStbxSI => CpuStbxSI,
          CpuWexSI => CpuWexSI,
          CpuAdrxDI => CpuAdrxDI,
          CpuDatxDI => CpuDatxDI,
          CpuDatxDO => CpuDatxDO,
          CpuAckxSO => CpuAckxSO,
          CpuErrxSO => CpuErrxSO,
          CpuRtyxSO => CpuRtyxSO,
          PhyTxClkxCI => PhyTxClkxCI,
          PhyTxdxDO => PhyTxdxDO,
          PhyTxEnxSO => PhyTxEnxSO,
          PhyTxErrxSO => PhyTxErrxSO,
          PhyRxClkxCI => PhyRxClkxCI,
          PhyRxdxDI => PhyRxdxDI,
          PhyRxDvxSI => PhyRxDvxSI,
          PhyRxErrxSI => PhyRxErrxSI,
          PhyColxSI => PhyColxSI,
          PhyCrsxSI => PhyCrsxSI,
          PhyMdcxSO => PhyMdcxSO,
          PhyMdioxDI => PhyMdioxDI,
          PhyMdioxDO => PhyMdioxDO,
          PhyMdioEnxSO => PhyMdioEnxSO,
          EthIntxSO => EthIntxSO
        );

	-- Clock process definitions
	ClkxCI_process :process
	begin
		ClkxCI <= '0';
		wait for ClkxCI_period/2;
		ClkxCI <= '1';
		wait for ClkxCI_period/2;
	end process;
	ClkxCIB <= not ClkxCI;
		 

	-- Stimulus process
	stim_proc: process
	begin		
		-- hold reset state for 100 ns.
		wait for 100 ns;	
		RstxRI	<= '0';

		wait until ClkxCI'event and ClkxCI='1';
		-- write mode register
		CpuWexSI <= '1';
		WbData 	<= x"00005403";
		WbAdr  	<= x"8000";	
		WbStart <= '1';
		wait until ClkxCI'event and ClkxCI='1' and WbDone='1';
		WbStart <= '0';
		wait until ClkxCI'event and ClkxCI='1';
		-- set mac address
		WbData 	<= OwnMacAdr(47 downto 16);
		WbAdr  	<= x"8040";	
		WbStart <= '1';
		wait until ClkxCI'event and ClkxCI='1' and WbDone='1';
		WbStart <= '0';
		wait until ClkxCI'event and ClkxCI='1';
		WbData 	<= x"0000" & OwnMacAdr(15 downto 0);
		WbAdr  	<= x"8044";	
		WbStart <= '1';
		wait until ClkxCI'event and ClkxCI='1' and WbDone='1';
		WbStart <= '0';
		wait until ClkxCI'event and ClkxCI='1';
		-- Note: we use sram starting at address 0 as data buffer for a dummy package
		-- set dest and src mac addresses
		WbData 	<= x"ffffffff";	-- first sram address holding the data package
		WbAdr  	<= x"0000";	
		WbStart <= '1';
		wait until ClkxCI'event and ClkxCI='1' and WbDone='1';
		WbStart <= '0';
		wait until ClkxCI'event and ClkxCI='1';
		WbData 	<= x"0201ffff";	
		WbAdr  	<= x"0004";	
		WbStart <= '1';
		wait until ClkxCI'event and ClkxCI='1' and WbDone='1';
		WbStart <= '0';
		wait until ClkxCI'event and ClkxCI='1';
		WbData 	<= x"06050403";	
		WbAdr  	<= x"0008";	
		WbStart <= '1';
		wait until ClkxCI'event and ClkxCI='1' and WbDone='1';
		WbStart <= '0';
		wait until ClkxCI'event and ClkxCI='1';
		-- add some dummy data into the package, just for testing
		WbData 	<= x"00abcdef";	
		WbAdr  	<= x"000c";	
		WbStart <= '1';
		wait until ClkxCI'event and ClkxCI='1' and WbDone='1';
		WbStart <= '0';
		wait until ClkxCI'event and ClkxCI='1';
		
		-- set tx buffer descriptor
		WbData 	<= x"00000000";	-- first sram address holding the data package
		WbAdr  	<= x"8404";	
		WbStart <= '1';
		wait until ClkxCI'event and ClkxCI='1' and WbDone='1';
		WbStart <= '0';
		wait until ClkxCI'event and ClkxCI='1';
		WbData 	<= x"0010b800"; 	-- mark buffer descriptor as read for transmission
		WbAdr  	<= x"8400";	
		WbStart <= '1';
		wait until ClkxCI'event and ClkxCI='1' and WbDone='1';
		WbStart <= '0';
		CpuWexSI <= '0';
		wait until ClkxCI'event and ClkxCI='1';
		
		-- read the buffer descriptor to see when the package was sent completely
		loop
			WbData 	<= x"00000000";
			WbAdr  	<= x"8400";	
			WbStart <= '1';
			wait until ClkxCI'event and ClkxCI='1' and WbDone='1';
			WbStart <= '0';
			wait until ClkxCI'event and ClkxCI='1';
			wait for 10 us;
		end loop;
		wait;
	end process;

	-- phy model
	eth_phy_inst : eth_phy 
	port map (
        -- COMMON
        m_rst_n_i	=> RstxRI,
		-- MAC TX
        mtx_clk_o	=> PhyTxClkxCI,
        mtxd_i		=> PhyTxdxDO,
        mtxen_i		=> PhyTxEnxSO,
        mtxerr_i	=> PhyTxErrxSO,
		-- MAC RX
        mrx_clk_o	=> PhyRxClkxCI,
        mrxd_o		=> PhyRxdxDI,
        mrxdv_o		=> PhyRxDvxSI,
        mrxerr_o	=> PhyRxErrxSI,
		-- common mii
        mcoll_o		=> PhyColxSI,
        mcrs_o		=> PhyCrsxSI,
        -- MIIM
        mdc_i		=> PhyMdcxSO,
        md_io		=> PhyMdioxD,
        -- SYSTEM
		phy_log		=> PhyLogxD
	);
	-- tristate buffer for Mdio pin of phy
	PhyMdioxDI <= PhyMdioxD;
	process (PhyMdioxDO, PhyMdioEnxSO)
	begin
		PhyMdioxD <= 'Z';
		if PhyMdioEnxSO='1' then
			PhyMdioxD <= PhyMdioxDO;
		end if;	
	end process;
	
	-- process doing 4 8bit wishbone writes to access a 32 bit slave on the data bus
	process 
	begin
		wait until ClkxCI'event and ClkxCI='1' and WbStart='1';
		CpuCycxSI	<= '1';
		CpuStbxSI	<= '1';
		--CpuWexSI	<= '1';
		CpuDatxDI	<= WbData(7 downto 0);
		CpuAdrxDI	<= WbAdr(15 downto 2) & "00";
		wait until ClkxCI'event and ClkxCI='1' and CpuAckxSO='1';
		CpuCycxSI	<= '1';
		CpuStbxSI	<= '1';
		--CpuWexSI	<= '1';
		CpuDatxDI	<= WbData(15 downto 8);
		CpuAdrxDI	<= WbAdr(15 downto 2) & "01";
		wait until ClkxCI'event and ClkxCI='1' and CpuAckxSO='1';
		CpuCycxSI	<= '1';
		CpuStbxSI	<= '1';
		--CpuWexSI	<= '1';
		CpuDatxDI	<= WbData(23 downto 16);
		CpuAdrxDI	<= WbAdr(15 downto 2) & "10";
		wait until ClkxCI'event and ClkxCI='1' and CpuAckxSO='1';
		CpuCycxSI	<= '1';
		CpuStbxSI	<= '1';
		--CpuWexSI	<= '1';
		CpuDatxDI	<= WbData(31 downto 24);
		CpuAdrxDI	<= WbAdr(15 downto 2) & "11";
		wait until ClkxCI'event and ClkxCI='1' and CpuAckxSO='1';
		CpuCycxSI	<= '0';
		CpuStbxSI	<= '0';
		--CpuWexSI	<= '0';
		CpuDatxDI	<= x"00";
		CpuAdrxDI	<= x"0000";
		WbDone		<= '1';
		wait until ClkxCI'event and ClkxCI='1' and WbStart='0';
		WbDone		<= '0';
	end process;

END;
