
LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
 
-- Uncomment the following library declaration if using
-- arithmetic functions with Signed or Unsigned values
--USE ieee.numeric_std.ALL;
 
ENTITY ioBusTb IS
END ioBusTb;
 
ARCHITECTURE behavior OF ioBusTb IS 
 
    -- Component Declaration for the Unit Under Test (UUT)
 
    COMPONENT ioBus
    PORT(
         ClkxCI : IN  std_logic;
         RstxRI : IN  std_logic;
         CpuCycxSI : IN  std_logic;
         CpuStbxSI : IN  std_logic;
         CpuWexSI : IN  std_logic;
         CpuAdrxDI : IN  std_logic_vector(5 downto 0);
         CpuDatxDI : IN  std_logic_vector(7 downto 0);
         CpuDatxDO : OUT  std_logic_vector(7 downto 0);
         CpuAckxSO : OUT  std_logic;
         CpuErrxSO : OUT  std_logic;
         CpuRtyxSO : OUT  std_logic;
         UartIntxSO : OUT  std_logic;
         SpiIntxSO : OUT  std_logic;
         UartTxxDO : OUT  std_logic;
         UartRxxDI : IN  std_logic;
         UartRtsxSO : OUT  std_logic;
         UartCtsxSI : IN  std_logic;
         SpiSckxSO : OUT  std_logic;
         SpiMisoxDI : IN  std_logic;
         SpiMosixDO : OUT  std_logic;
         SpiSSxSO : OUT  std_logic_vector(2 downto 0);
         SwitchxDI : IN  std_logic_vector(3 downto 0);
         ButtonxDI : IN  std_logic_vector(3 downto 0);
         LedxDO : OUT  std_logic_vector(7 downto 0)
        );
    END COMPONENT;
    

   --Inputs
   signal ClkxCI : std_logic := '0';
   signal RstxRI : std_logic := '1';
   signal CpuCycxSI : std_logic := '0';
   signal CpuStbxSI : std_logic := '0';
   signal CpuWexSI : std_logic := '0';
   signal CpuAdrxDI : std_logic_vector(5 downto 0) := (others => '0');
   signal CpuDatxDI : std_logic_vector(7 downto 0) := (others => '0');
   signal UartRxxDI : std_logic := '0';
   signal UartCtsxSI : std_logic := '0';
   signal SpiMisoxDI : std_logic := '0';
   signal SwitchxDI : std_logic_vector(3 downto 0) := (others => '0');
   signal ButtonxDI : std_logic_vector(3 downto 0) := (others => '0');

 	--Outputs
   signal CpuDatxDO : std_logic_vector(7 downto 0);
   signal CpuAckxSO : std_logic;
   signal CpuErrxSO : std_logic;
   signal CpuRtyxSO : std_logic;
   signal UartIntxSO : std_logic;
   signal SpiIntxSO : std_logic;
   signal UartTxxDO : std_logic;
   signal UartRtsxSO : std_logic;
   signal SpiSckxSO : std_logic;
   signal SpiMosixDO : std_logic;
   signal SpiSSxSO : std_logic_vector(2 downto 0);
   signal LedxDO : std_logic_vector(7 downto 0);

   -- Clock period definitions
   constant ClkxCI_period : time := 25 ns;
 
BEGIN
 
	-- Instantiate the Unit Under Test (UUT)
	uut: ioBus PORT MAP (
		  ClkxCI => ClkxCI,
          RstxRI => RstxRI,
          CpuCycxSI => CpuCycxSI,
          CpuStbxSI => CpuStbxSI,
          CpuWexSI => CpuWexSI,
          CpuAdrxDI => CpuAdrxDI,
          CpuDatxDI => CpuDatxDI,
          CpuDatxDO => CpuDatxDO,
          CpuAckxSO => CpuAckxSO,
          CpuErrxSO => CpuErrxSO,
          CpuRtyxSO => CpuRtyxSO,
          UartIntxSO => UartIntxSO,
          SpiIntxSO => SpiIntxSO,
          UartTxxDO => UartTxxDO,
          UartRxxDI => UartRxxDI,
          UartRtsxSO => UartRtsxSO,
          UartCtsxSI => UartCtsxSI,
          SpiSckxSO => SpiSckxSO,
          SpiMisoxDI => SpiMisoxDI,
          SpiMosixDO => SpiMosixDO,
          SpiSSxSO => SpiSSxSO,
          SwitchxDI => SwitchxDI,
          ButtonxDI => ButtonxDI,
          LedxDO => LedxDO
	);

	-- Clock process definitions
	ClkxCI_process :process
	begin
		ClkxCI <= '0';
		wait for ClkxCI_period/2;
		ClkxCI <= '1';
		wait for ClkxCI_period/2;
	end process;
 

	-- Stimulus process
	stim_proc: process
	begin		
		wait for 100 ns;
		RstxRI <= '0';
	  
		-- test spi interface 
		wait until ClkxCI'event and ClkxCI='1';
		CpuAdrxDI	<= "000100";	-- SPCR
		CpuDatxDI	<= x"D1";		-- SPIE, SPI, MSTR, SPR = 01
		CpuWexSI	<= '1';
		CpuCycxSI	<= '1';
		CpuStbxSI	<= '1';
		wait until ClkxCI'event and ClkxCI='1' and (CpuAckxSO='1' or CpuErrxSO='1');
		CpuCycxSI	<= '0';
		CpuStbxSI	<= '0';
		
		wait until ClkxCI'event and ClkxCI='1';
		CpuAdrxDI	<= "000110";	-- SPDR
		CpuDatxDI	<= x"A5";		-- send this byte
		CpuWexSI	<= '1';
		CpuCycxSI	<= '1';
		CpuStbxSI	<= '1';
		wait until ClkxCI'event and ClkxCI='1' and (CpuAckxSO='1' or CpuErrxSO='1');
		CpuCycxSI	<= '0';
		CpuStbxSI	<= '0';
		
		wait until ClkxCI'event and ClkxCI='1';
		CpuAdrxDI	<= "000101";	-- read SPSR
		CpuDatxDI	<= x"00";		
		CpuWexSI	<= '0';
		CpuCycxSI	<= '1';
		CpuStbxSI	<= '1';
		wait until ClkxCI'event and ClkxCI='1' and (CpuAckxSO='1' or CpuErrxSO='1');
		CpuCycxSI	<= '0';
		CpuStbxSI	<= '0';
		
		wait for 40*ClkxCI_period;

		wait until ClkxCI'event and ClkxCI='1';
		CpuAdrxDI	<= "000101";	-- read SPSR
		CpuDatxDI	<= x"00";		
		CpuWexSI	<= '0';
		CpuCycxSI	<= '1';
		CpuStbxSI	<= '1';
		wait until ClkxCI'event and ClkxCI='1' and (CpuAckxSO='1' or CpuErrxSO='1');
		CpuCycxSI	<= '0';
		CpuStbxSI	<= '0';
		

		wait;
	end process;

END;
