--/**************************************************************************************************************
--*
--*   B i t H o u n d   -   A n   F P G A   B a s e d   L o g i c   A n a l y z e r
--*
--*   FPGA Design 
--* 
--* Copyright 2010-2011   Mario Mauerer (MM), Lukas Schrittwieser (LS), ETH Zurich
--*
--*    This program is free software: you can redistribute it and/or modify
--*    it under the terms of the GNU General Public License as published by
--*    the Free Software Foundation, either version 3 of the License, or
--*    (at your option) any later version.
--*
--*    This program is distributed in the hope that it will be useful,
--*    but WITHOUT ANY WARRANTY; without even the implied warranty of
--*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
--*    GNU General Public License for more details.
--*
--*    You should have received a copy of the GNU General Public License
--*    along with this program.  If not, see <http://www.gnu.org/licenses/>.
--*
--***************************************************************************************************************
--*
--* Change Log:
--*
--* Version 1.0 - 2011/06/15 - LS
--*   first release
--*
--*
--***************************************************************************************************************
--*
--* Namin convention:  http://dz.ee.ethz.ch/en/information/hdl-help/vhdl-naming-conventions.html
--*
--***************************************************************************************************************
--*
--* This is the testbench for sync.vhd. It is a simple testbench with hardcoded
--* stimuli for visual inspection in a simulator.
--*
--***************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;

entity sync_tb is
end sync_tb;


architecture tb of sync_tb is

  component sync
    generic (
      CHANNELS : integer);
    port (
      ClkxCI      : in  std_logic;
      ChannelsxDI : in  std_logic_vector (CHANNELS-1 downto 0);
      FilterxS    : in  std_logic;
      DemuxxS     : in  std_logic;
      FallingxS   : in  std_logic;
      SyncedxDO   : out std_logic_vector (CHANNELS-1 downto 0));
  end component;

  -- component generics
  constant CHANNELS : integer := 32;

  -- component ports
  signal ClkxCI      : std_logic := '0';
  signal ChannelsxDI : std_logic_vector (CHANNELS-1 downto 0) := (others => '0');
  signal FilterxS    : std_logic := '0';
  signal DemuxxS     : std_logic := '0';
  signal FallingxS   : std_logic := '0';
  signal SyncedxDO   : std_logic_vector (CHANNELS-1 downto 0);

  -- clock
  signal Clk : std_logic := '1';

begin  -- tb

  -- component instantiation
  DUT: sync
    generic map (
      CHANNELS => CHANNELS)
    port map (
      ClkxCI      => ClkxCI,
      ChannelsxDI => ChannelsxDI,
      FilterxS    => FilterxS,
      DemuxxS     => DemuxxS,
      FallingxS   => FallingxS,
      SyncedxDO   => SyncedxDO);

  -- clock generation
  ClkxCI <= not ClkxCI after 10 ns;

  -- waveform generation
  WaveGen_Proc: process
  begin
    -- first: std syncing with no flags set
    wait for 5 ns;
    ChannelsxDI <= x"01234567";
    wait for 20 ns;
    ChannelsxDI <= x"00000000";
    wait for 20 ns;
    ChannelsxDI <= x"12345678";
    wait for 20 ns;
    ChannelsxDI <= x"9abcdef0";
    wait for 20 ns;

    -- demulitplexed mode
    wait until ClkxCI'event and ClkxCI = '1';
	wait until ClkxCI'event and ClkxCI = '1';
	wait until ClkxCI'event and ClkxCI = '1';
	wait until ClkxCI'event and ClkxCI = '1';
    DemuxxS <= '1';
	ChannelsxDI <= x"00000000";
	wait until ClkxCI'event and ClkxCI = '1';
	wait for 5 ns;
    ChannelsxDI <= x"00001234";
    wait for 10 ns;
    ChannelsxDI <= x"00005678";
    wait for 10 ns;
    ChannelsxDI <= x"00009abc";
    wait for 10 ns;
    ChannelsxDI <= x"0000def0";
    wait for 10 ns;

    -- falling edge mode
    wait until ClkxCI'event and ClkxCI = '1';
	wait until ClkxCI'event and ClkxCI = '1';
	wait until ClkxCI'event and ClkxCI = '1';
	wait until ClkxCI'event and ClkxCI = '1';
	DemuxxS <= '0';
    FallingxS <= '1';
	ChannelsxDI <= x"00000000";
	wait until ClkxCI'event and ClkxCI = '1';
    wait for 5 ns;
    ChannelsxDI <= x"00000001";
    wait for 10 ns;
    ChannelsxDI <= x"00000002";
    wait for 10 ns;
    ChannelsxDI <= x"00000003";
    wait for 10 ns;
    ChannelsxDI <= x"00000003";
    wait for 10 ns;
    ChannelsxDI <= x"00000004";
    wait for 10 ns;
    ChannelsxDI <= x"00000005";
    wait for 10 ns;
    ChannelsxDI <= x"00000006";
    wait for 10 ns;

    -- filtered mode
    wait until ClkxCI'event and ClkxCI = '1';
    wait until ClkxCI'event and ClkxCI = '1';
	wait until ClkxCI'event and ClkxCI = '1';
	wait until ClkxCI'event and ClkxCI = '1';
	FallingxS <= '0';
    FilterxS <= '1';
	ChannelsxDI <= x"00000000";
	wait until ClkxCI'event and ClkxCI = '1';
	wait for 5 ns;
    ChannelsxDI <= x"00000000";         -- falling edge
    wait for 10 ns;
    ChannelsxDI <= x"00000001";         -- rising edge, glitch
    wait for 10 ns;
    ChannelsxDI <= x"00000000";
    wait for 10 ns;
    ChannelsxDI <= x"00000001";
    wait for 10 ns;
    ChannelsxDI <= x"00000001";
    wait for 10 ns;
    ChannelsxDI <= x"00000001";
    wait for 10 ns;
    ChannelsxDI <= x"00000000";
    wait for 10 ns;
    
    ChannelsxDI <= (others => '0');
	wait;
	
  end process WaveGen_Proc;

  

end tb;

